from logging import critical
from networkx import DiGraph
from generalized_geography.common.utils import split_nodes_by_type
from generalized_geography.datasets import load_oldict_words
import generalized_geography as gg

words = load_oldict_words()

g = DiGraph()
for word in words:
    start_node = word[0]
    end_node = word[-1]
    if g.has_edge(start_node, end_node):
        g[start_node][end_node]["num"] += 1
    else:
        g.add_edge(start_node, end_node, num=1)

result = gg.deg_classify_fast(g)
lose, win, loop = split_nodes_by_type(result.type_map)
# print("Lose:", lose)
# print("Win:", win)
# print("Loop:", loop)

critical_edges = gg.deg_get_critical_edges(result.pruned_graph)
print("Critical edges:", critical_edges)



from networkx import edges, maximum_flow, maximum_flow_value, strongly_connected_components
import generalized_geography as gg


graph = gg.load_guel_graph()

result = gg.cdeg_classify_fast(graph)
unk_graph = result.pruned_graph
win, lose, loop = gg.split_nodes_by_type(result.type_map)
win = gg.nodes_in_pos(win, 0)
lose = gg.nodes_in_pos(lose, 0)
loop = gg.nodes_in_pos(loop, 0)

capacity_graph = gg.make_capacity_graph(unk_graph)

moves = [(u, v, data["num"])
         for u, v, data in unk_graph.edges(data=True) if u.pos == 1]

scc_list = list(strongly_connected_components(unk_graph))

# 노드 -> SCC 인덱스 맵 생성
node_to_scc = {}
for idx, scc in enumerate(scc_list):
    for node in scc:
        node_to_scc[node] = idx

for u, v, num in moves:
    # u,v가 동일한 강연결요소에 속하는지 확인
    if node_to_scc[u] != node_to_scc[v]:
        continue

    flow_num, flow_dict = maximum_flow(
        capacity_graph, v, u)
    flow_num += 1
    if num > flow_num:

        print(u.name, "->", v.name, num, flow_num)
        bottlenecks = []
        for u, v, data in capacity_graph.edges(data=True):
            cap = data['capacity']
            f = flow_dict[u].get(v, 0)
            if f == cap and f > 0:
                bottlenecks.append((u, v, cap))
        print("병목 간선:", [(u.name, v.name) for u, v, _ in bottlenecks])



from matplotlib import pyplot as plt
from networkx.algorithms.cycles import simple_cycles
from networkx import DiGraph, draw, find_cycle
import generalized_geography as gg
import networkx as nx
import random

n = 5  # 노드 개수
G = nx.complete_graph(n)

# 방향 그래프로 변환
DG = nx.DiGraph()
DG.add_nodes_from(G.nodes())


# ✅ 짝수 사이클 강제로 하나 추가 (예: 0 → 1 → 2 → 3 → 0, 길이 4)
even_cycle_nodes = [0, 1, 2, 3]  # 길이 4 (짝수)
for i in range(len(even_cycle_nodes)):
    u = even_cycle_nodes[i]
    v = even_cycle_nodes[(i + 1) % len(even_cycle_nodes)]
    DG.add_edge(u, v, num=random.randint(1, 3))  # 방향 강제 지정

# 각 간선마다 무작위 방향 지정
for u, v in G.edges():
    if DG.has_edge(u, v) or DG.has_edge(v, u):
        continue  # 이미 간선이 추가된 경우 건너뜀
    if random.random() < 0.5:
        DG.add_edge(u, v, num=random.randint(1, 3))
    else:
        DG.add_edge(v, u, num=random.randint(1, 3))


# 이제 짝수 사이클을 찾음
cycle = [(v, even_cycle_nodes[(i + 1) % len(even_cycle_nodes)])
         for i, v in enumerate(even_cycle_nodes)]

# 혹시 find_cycle이 홀수 사이클을 반환하면,
# 짝수 사이클만 필터링


# ------------------------------------------------
# 이후 부분 동일
increase_limit = 100
print("cycle:", [u for u in cycle])
for i in range(increase_limit):
    for u, v in cycle:
        DG[u][v]['num'] = i
    type_map = gg.deg_classify_complete(DG)

    print("iteration:", i, "type_map:", "".join(
        [str(type_map[j]) for j in range(n)]))




import generalized_geography as gg
from generalized_geography.types.bipartite import BipartiteNode


graph = gg.load_guel_graph()
graph.remove_edge(BipartiteNode("첩", 1), BipartiteNode("꽂", 0))
result = gg.cdeg_classify_fast(graph)
unk_graph = result.pruned_graph

win, lose, loop = gg.split_nodes_by_type(result.type_map)
win = gg.nodes_in_pos(win, 0)
lose = gg.nodes_in_pos(lose, 0)
loop = gg.nodes_in_pos(loop, 0)

critical_edges = gg.cdeg_get_critical_edges(unk_graph)
print("Critical edges:", "\n".join(
    sorted([u.name+v.name for u, v in critical_edges])))



import generalized_geography as gg
import networkx as nx


graph = nx.DiGraph()
graph.add_edges_from([(1, 2, {"num": 2}), (2, 3, {"num": 5}), (3, 1, {
                     "num": 1}), (3, 4, {"num": 1}), (4, 4, {"num": 5})])
result = gg.deg_classify_fast(graph)
lose, win = gg.split_nodes_by_resolved_type(result.type_map)
print("Lose nodes:", lose)
print("Win nodes:", win)
print("Unknown nodes:", result.pruned_graph.nodes)
type_map = gg.deg_classify_complete(graph)
print()
print("After complete classification:")
lose, win = gg.split_nodes_by_resolved_type(type_map)
print("Lose nodes:", lose)
print("Win nodes:", win)
